EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'NuGetPackagesV2_CreateOrUpdatePackage',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new NuGet package or updates an existing one.'
GO

IF OBJECT_ID('[NuGetPackagesV2_CreateOrUpdatePackage]') IS NOT NULL DROP PROCEDURE [NuGetPackagesV2_CreateOrUpdatePackage]
GO

SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

CREATE PROCEDURE [NuGetPackagesV2_CreateOrUpdatePackage]
(
	@Feed_Id INT,
	@Package_Id VARCHAR(255),
	@Version_Text VARCHAR(255),
	@FullVersion_Text NVARCHAR(255),
	@Published_Date DATETIME,
	@Package_Size BIGINT,
	@PackageHash_SHA1_Bytes BINARY(20),
	@PackageHash_SHA512_Bytes BINARY(64),
	@TargetFrameworks_Text VARCHAR(255),
	@Nuspec_Bytes VARBINARY(MAX),
	@Symbols_Indicator YNINDICATOR,
	@Source_Indicator YNINDICATOR,
	@Cached_Indicator YNINDICATOR,
	@Listed_Indicator YNINDICATOR,
	@Download_Count INT = NULL,
	@LatestVersion_Text VARCHAR(255) = NULL,
	@LatestStableVersion_Text VARCHAR(255) = NULL,
	@SymbolFiles_Xml XML = NULL,
	@NuGetPackageV2_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON
SET QUOTED_IDENTIFIER ON

BEGIN TRY
	BEGIN TRANSACTION

		SET @NuGetPackageV2_Id = (SELECT [NuGetPackageV2_Id]
									FROM [NuGetPackagesV2]
									WITH (TABLOCKX)
								   WHERE [Feed_Id] = @Feed_Id
									 AND [Package_Id] = @Package_Id)

		IF @NuGetPackageV2_Id IS NULL BEGIN
			INSERT INTO [NuGetPackagesV2]
			(
				[Feed_Id],
				[Package_Id],
				[LatestVersion_Text],
				[LatestStableVersion_Text]
			)
			VALUES
			(
				@Feed_Id,
				@Package_Id,
				@LatestVersion_Text,
				@LatestStableVersion_Text
			)
			SET @NuGetPackageV2_Id = SCOPE_IDENTITY()
		END ELSE IF @LatestVersion_Text IS NOT NULL BEGIN
			UPDATE [NuGetPackagesV2]
			   SET [LatestVersion_Text] = @LatestVersion_Text,
			       [LatestStableVersion_Text] = COALESCE(@LatestStableVersion_Text, [LatestStableVersion_Text])
			 WHERE [NuGetPackageV2_Id] = @NuGetPackageV2_Id
		END

		IF EXISTS(SELECT * FROM [NuGetPackageVersionsV2] WHERE [NuGetPackageV2_Id] = @NuGetPackageV2_Id and [Version_Text] = @Version_Text) BEGIN

			UPDATE [NuGetPackageVersionsV2]
			   SET [FullVersion_Text] = COALESCE(@FullVersion_Text, [FullVersion_Text]),
				   [Published_Date] = COALESCE(@Published_Date, [Published_Date]),
				   [Package_Size] = COALESCE(@Package_Size, [Package_Size]),
				   [PackageHash_SHA1_Bytes] = COALESCE(@PackageHash_SHA1_Bytes, [PackageHash_SHA1_Bytes]),
				   [PackageHash_SHA512_Bytes] = COALESCE(@PackageHash_SHA512_Bytes, [PackageHash_SHA512_Bytes]),
				   [TargetFrameworks_Text] = COALESCE(@TargetFrameworks_Text, [TargetFrameworks_Text]),
				   [Nuspec_Bytes] = COALESCE(@Nuspec_Bytes, [Nuspec_Bytes]),
				   [Symbols_Indicator] = COALESCE(@Symbols_Indicator, [Symbols_Indicator]),
				   [Source_Indicator] = COALESCE(@Source_Indicator, [Source_Indicator]),
				   [Cached_Indicator] = COALESCE(@Cached_Indicator, [Cached_Indicator]),
				   [Listed_Indicator] = COALESCE(@Listed_Indicator, [Listed_Indicator]),
				   [Download_Count] = COALESCE(@Download_Count, [Download_Count])
			 WHERE [NuGetPackageV2_Id] = @NuGetPackageV2_Id
			   AND [Version_Text] = @Version_Text

		END ELSE BEGIN

			INSERT INTO [NuGetPackageVersionsV2]
			(
				[NuGetPackageV2_Id],
				[Version_Text],
				[FullVersion_Text],
				[Published_Date],
				[Package_Size],
				[Download_Count],
				[PackageHash_SHA1_Bytes],
				[PackageHash_SHA512_Bytes],
				[TargetFrameworks_Text],
				[Nuspec_Bytes],
				[Symbols_Indicator],
				[Source_Indicator],
				[Cached_Indicator],
				[Listed_Indicator]
			)
			VALUES
			(
				@NuGetPackageV2_Id,
				@Version_Text,
				@FullVersion_Text,
				@Published_Date,
				@Package_Size,
				COALESCE(@Download_Count, 0),
				@PackageHash_SHA1_Bytes,
				@PackageHash_SHA512_Bytes,
				@TargetFrameworks_Text,
				@Nuspec_Bytes,
				COALESCE(@Symbols_Indicator, 'N'),
				COALESCE(@Source_Indicator, 'N'),
				COALESCE(@Cached_Indicator, 'N'),
				COALESCE(@Listed_Indicator, 'Y')
			)

		END

		DELETE FROM [NuGetPackageSymbolsV2]
		 WHERE [Feed_Id] = @Feed_Id
		   AND [NuGetPackageV2_Id] = @NuGetPackageV2_Id
		   AND [Package_Version_Text] = @Version_Text

		IF @Symbols_Indicator = 'Y'
		BEGIN
			INSERT INTO [NuGetPackageSymbolsV2]
			(
				[Feed_Id],
				[Symbol_Id],
				[SymbolFileName_Text],
				[Package_SymbolFilePath_Text],
				[Symbol_Age],
				[NuGetPackageV2_Id],
				[Package_Version_Text]
			)
			SELECT @Feed_Id,
			       SFX.[SymbolFile].[value]('@Id', 'VARBINARY(20)'),
			       SFX.[SymbolFile].[value]('@FileName', 'NVARCHAR(255)'),
			       SFX.[SymbolFile].[value]('@Path', 'NVARCHAR(255)'),
			       SFX.[SymbolFile].[value]('@Age', 'INT'),
			       @NuGetPackageV2_Id,
			       @Version_Text
			  FROM @SymbolFiles_Xml.[nodes]('/SymbolFiles/SymbolFile') AS SFX([SymbolFile])
			
		END


		EXEC [Events_RaiseEvent] 'PKGADD', @Feed_Id, 'Package_Name', @Package_Id, 'Package_Version', @Version_Text

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [NuGetPackagesV2_CreateOrUpdatePackage] TO [ProGetUser_Role]